package es.uva.davplov.tfg_patronlibre;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Path;
import android.util.AttributeSet;
import android.view.MotionEvent;
import android.view.View;
import android.widget.Toast;


public class DrawView extends View {

	private long lastTime;
	private List<Point> puntos= new ArrayList<Point>();
    private long time = 0;
	
	
	private Paint mPaint;
	
	public List<Point> getPuntos() {
		return this.puntos;
	}

	public void setPuntos(List<Point> puntos) {
		this.puntos = puntos;
	}

	private Path mPath;

    public DrawView(Context context)
    {
        super(context);
        init();
    }
   
    public DrawView(Context context, AttributeSet attrs)
    {
        super(context, attrs);
        init();
    }
    

    public DrawView(Context context, AttributeSet attrs, int defStyle) {
        super(context, attrs, defStyle);
        init();
    }
    
 
    private void init(){
		mPaint = new Paint();
		mPath = new Path();

        mPaint.setAntiAlias(true);
        mPaint.setDither(true);
        mPaint.setColor(Color.WHITE);
        mPaint.setStyle(Paint.Style.STROKE);
        mPaint.setStrokeJoin(Paint.Join.ROUND);
        mPaint.setStrokeWidth(4f);

		// to make it focusable so that it will receive touch events properly
		setFocusable(true);
    }


	public void clear() {
		puntos= new ArrayList<Point>();
		lastTime=0;
		mPath.reset();
		invalidate();
	}

	@Override
	protected void onDraw(Canvas canvas) {
		canvas.drawPath(mPath, mPaint);
	}
	
	@Override
	public boolean onTouchEvent(MotionEvent event) {
				
		
		if (!(lastTime == 0)) {
			time = Calendar.getInstance().getTimeInMillis() - lastTime;
		}
		final float x = (int) event.getX();
		final float y = (int) event.getY();
		
		
		/*En este switch se determina si es la primera pulsacin en el DrawView o no*/
		
		switch (event.getAction()) {
		
		//Si es la primera vez que se pulsa
		case MotionEvent.ACTION_DOWN:
			mPath.moveTo(x, y);
			puntos.add(new Point(x, y, time, 1));
						
			return true;
		case MotionEvent.ACTION_MOVE:
			
			mPath.lineTo(x, y);
			
		case MotionEvent.ACTION_UP:
			puntos.add(new Point(x, y, time, 0));
			break;
		}
		lastTime=Calendar.getInstance().getTimeInMillis();
		invalidate(); // calls onDraw method
		return true;
	}
}