package featureCalculation;

import java.util.List;

import srcUmbral.configuracion.Configuracion;
import srcUmbral.firmweb.util.ArraysUtil;
import srcUmbral.firmweb.util.Matematicas;

import es.uva.davplov.tfg_patronlibre.Point;

/**
 * 
 * @author David Polvorosa
 * Clase que permite la conversion de las muestras a un formato que se puede usar en fastDTW
 *
 */
public class FeatureCalcultation {

	int numCaracPoint = Point.NUM_CARAC_POINT;

	public double[][] featureCalculation(List<Point> patronOrig) {

		double[][] patron = new double[patronOrig.size()][numCaracPoint];
		double t = (double) ((Point) patronOrig.get(0)).getT();

		for (int i = 0; i < patronOrig.size(); i++) {
			patron[i][0] = (double) ((Point) patronOrig.get(i)).getX();
			patron[i][1] = (double) ((Point) patronOrig.get(i)).getY();
			t += (double) ((Point) patronOrig.get(i)).getT();
			patron[i][2] = t;
		}
		
		//Normalizamos respeto al eje de coordenadas
		
		patron = centerize(patron);
		
		//Normalizamos respecto al tamao de la muestra
		patron = znorm(patron);
		return patron;
	}
	
	/**
	 * Normaliza los datos recibidos mediante la norma-z
	 * A diferencia de la otra, normaliza todas las columnas de la matriz de datos
	 * 
	 * @param datos
	 */
	private static double[][] znorm(double[][] datos) {
		double[][] znorm = ArraysUtil.copy(datos);
		for (int col = 0; col < datos[0].length; col++) {
			double mean = Matematicas.getMean(datos, col);
			double desv = Matematicas.getDesvStd(datos, col);
			desv = (desv == 0) ? 1 : desv; // Para evitar division por cero

			// Normalizamos toda la columna de la matriz
			for (int i = 0; i < datos.length; i++) {
				znorm[i][col] = (datos[i][col] - mean) / desv;
			}
		}
		return znorm;
	}
	
	/**
	 * 
	 * @param datosTableta
	 * @return
	 */
	private static double[][] centerize(double[][] datosTableta) {

		double[][] ddatosTableta = ArraysUtil.copy(datosTableta);

		// Centro de masa de las coordenadas X e Y
		final int X = 0, Y = 1;
		double cmX, cmY;
		if (Configuracion.NormCMFMod1) {
			cmX = Configuracion.CmXFMod1;
			cmY = Configuracion.CmYFMod1;
		} else {
		 cmX = Matematicas.getMean(datosTableta, X);
		 cmY = Matematicas.getMean(datosTableta, Y);
		}
		for (int i = 0; i < datosTableta.length; i++) {
			ddatosTableta[i][X] = datosTableta[i][X] - cmX;
			ddatosTableta[i][Y] = datosTableta[i][Y] - cmY;
		}
		return ddatosTableta;
	}

}
