package srcPruebaSeguridad;

import java.util.LinkedHashMap;

import android.util.Log;
import fastDTW.dtw.TimeWarpInfo;
import fastDTW.timeseries.TimeSeries;
import srcUmbral.MisExcepciones.UmbralException;
import srcUmbral.configuracion.Configuracion;
import srcUmbral.firmweb.util.Matematicas;
import srcUmbral.umbral.Eer;

public class PruebaSeguridad {

	/**
	 * @param args
	 */
	public double pruebaSeguridad(double [][][] CaracFirmasEnt, double [][][] CaracFirmasDev) throws UmbralException {

		// Calculamos las salidas para muestras de desarrollo
		double[] dScoTplDev = null;
		dScoTplDev = this.testCohort(CaracFirmasEnt, CaracFirmasDev);

		// Calculamos salidas propias mediante leave-one-out
		double[] dScoAut = this.leaveOneOut(CaracFirmasEnt);
		
		return calculaEER(dScoTplDev,dScoAut);

	}
	
	private double[] testCohort(double[][][] firmasEnt, double[][][] fichDev)
			throws UmbralException {
		 
		
		double[] salidasCohort = new double[fichDev.length];
		for (int i = 0; i < fichDev.length; i++) {
			Log.d("Ficheros", "Comparo con el cohorte "+(1+i));
			double[][] caracFirma = fichDev[i];
			// Ahora sacamos el score al comparar con cada firma de inscripcion
			double[] scores = new double[firmasEnt.length];
			for (int j = 0; j < firmasEnt.length; j++) {
				// Firma de ins
				double[][] firmaEntPro = firmasEnt[j];
				// Calculamos la distancia DTW entre ambas
				//double dtwDist = Dtw.dtwDistance(firmaEntPro, caracFirma);
				TimeSeries firmaEnt = new TimeSeries(firmaEntPro, false);
				TimeSeries firmaCh = new TimeSeries(caracFirma, false);
				TimeWarpInfo info = fastDTW.dtw.FastDTW.getWarpInfoBetween(firmaEnt,
						firmaCh, Configuracion.radio, Configuracion.distFn);
				double dtwDist = info.getDistance();
				// Originaldouble dtwDist = Dtw.distance(firmaEntPro, caracFirma);
				// La almacenamos en el array de scores
				scores[j] = dtwDist;
			}
			// Calculamos la distancia m�n. de la firma de prueba a las de
			// inscrip.
			double minScore = Matematicas.min(scores);
			// Aadimos al array final
			salidasCohort[i] = minScore;
		}
		return salidasCohort;
	}
	
	private double[] leaveOneOut(double[][][] caracFirmasEnt) {
		double[] scoAut = new double[caracFirmasEnt.length];
		for (int i = 0; i < caracFirmasEnt.length; i++) {
			// Firma actual (referencia)
			double[][] ref = caracFirmasEnt[i];
			// Comparamos con el resto de firmas de la inscripcion
			double[] scoTemp = new double[caracFirmasEnt.length - 1];
			int indice = 0;
			for (int j = 0; j < caracFirmasEnt.length; j++) {
				// Saltamos la comparaci�n consigo misma
				if (i == j)
					continue;
				// Firma de prueba
				double[][] test = caracFirmasEnt[j];
				// Calculamos la distancia DTW entre ambas
				//double dtwDist = Dtw.dtwDistance(ref, test);
				//Original: double dtwDist = Dtw.distance(ref, test);
				TimeSeries firmaRef = new TimeSeries(ref, false);
				TimeSeries firmaTest = new TimeSeries(test, false);
				TimeWarpInfo info = fastDTW.dtw.FastDTW.getWarpInfoBetween(firmaRef,
						firmaTest, Configuracion.radio, Configuracion.distFn);
				double dtwDist = info.getDistance();
				// La almacenamos en el array de distancias
				scoTemp[indice] = dtwDist;
				indice++;
			}
			scoAut[i] = Matematicas.min(scoTemp);
		}
		return scoAut;
	}
	
	private double calculaEER(double[] devs, double[] plns) throws UmbralException {
		
		// Calculo EER
		Eer eerPriori = Eer.getEer(plns, devs);
		return eerPriori.eerValue;
		
	}

}
