package srcUmbral.firmweb.util;

import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FilenameFilter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.StringTokenizer;

import srcUmbral.configuracion.Configuracion;

import srcUmbral.MisExcepciones.UmbralException;

public class FileUtil {

	/**
	 * 
	 * @param args
	 */
	public static void main(String[] args) throws UmbralException {

		String fichero = "D:/temp/FirmWeb/cevp2/cevp26.dat";
		double[][] datos = leeFicheroFirma(fichero);
		for (int i = 0; i < datos.length; i++) {

			// if ((i + 1) != datos.length && datos[i][0] == datos[i + 1][0])
			// continue;
			System.out.print(i + 1 + "\t");
			for (int j = 1; j < datos[i].length; j++) {
				System.out.print(datos[i][j] + "\t");
			}
			System.out.println();
		}

	}

	/**
	 * Devuelve el n�mero de l�neas del fichero de la firma
	 * 
	 * @param fichero
	 * @return
	 * @throws UmbralException 
	 */
	private static int getNumLineasAsciiFile(String fichero) throws UmbralException {
		int numLineas = 0;
		try {
			BufferedReader br = new BufferedReader(new FileReader(fichero));
			while (br.ready()) {
				String line = br.readLine();
				//Original if (line != null && line.split("\t").length == 6) {
                                if (line != null) {
					numLineas++;
				} else {
					break;
				}
			}
			br.close();
		} catch (IOException e) {
			throw new UmbralException ("ERROR al ejecutar getNumLineasAsciiFile sobre " + fichero + ". Exception: " + e.toString());
			//e.printStackTrace();
		}
		return numLineas;

	}


	/**
	 * Lee un fichero de firma independientemente del formato
	 * 
	 * @param fichero
	 * @return
	 */
	public static double[][] leeFicheroFirma(String fichero) throws UmbralException {

		double[][] txypai = null;

		// Si la extension del fichero es .ascii lo leemos en modo ASCII
		if (fichero.endsWith("ascii")) {
			txypai = readAsciiFile(fichero);
			Configuracion.tipoFichero = "ascii";
		}

		// Si la extension es .dat es binario
		else if (fichero.endsWith("dat")) {
			txypai = readBinaryFile(fichero);
			Configuracion.tipoFichero = "dat";
		}

                // Si la extension es .txt es el nuevo de LoginBySign
		else if (fichero.endsWith("txt")) {
			txypai = readTxtFile(fichero);
			Configuracion.tipoFichero = "txt";
		}
		
		// Extension de los ficheros del corpus. Formato que vamos a usar aqui
		else if (fichero.endsWith("frm")) {
			txypai = readFrmFile(fichero);
			Configuracion.tipoFichero = "frm";
		}

		// Devolvemos lo le�do
		return txypai;

	}

	private static double[][] readFrmFile(String fichero) throws UmbralException {
		//  MUY IMPORTANTE: EL ORDEN DE LOS DATOS ES DIFERENTE. EN ESTOS ES XYTPAI Y EN LOS
		//QUE SE LEIAN HASTA AHORA ERA TXYPAI. LO LEO PASANDO EL TIEMPO AL PRINCIPIO PARA NO
		//MODIFICAR EL RESTO DEL PROGRAMA...
		
		// Vemos las lineas del fichero
		int nl = getNumLineasAsciiFile(fichero);

		// Creamos el array de enteros
		double[][] datos = new double[nl][6];

		// Si el fichero existe lo leemos
		BufferedReader brFile = null;
		try {

			// Abrimos el fichero a leer y lo leemos
			brFile = new BufferedReader(new FileReader(fichero));
			int fila = 0;
			while (brFile.ready()) {

				// Pasamos la lnea en array de doubles
				String lineData = brFile.readLine();
				if (lineData == null)
					continue;

				// Como la linea contiene numero y texto la leemos token a token
				StringTokenizer lineDataTok = new StringTokenizer(lineData);
				//leemos X
				if (lineDataTok.hasMoreTokens()) {
					datos[fila][1] = Double.parseDouble(lineDataTok.nextToken());
				} else {
					return null;
				}
				//leemos Y
				if (lineDataTok.hasMoreTokens()) {
					datos[fila][2] = Double.parseDouble(lineDataTok.nextToken());
				} else {
					return null;
				}
				//leemos t
				if (lineDataTok.hasMoreTokens()) {
					datos[fila][0] = Double.parseDouble(lineDataTok.nextToken());
				} else {
					return null;
				}
				//leemos P
				if (lineDataTok.hasMoreTokens()) {
					if (lineDataTok.nextToken().equals("false")) {
						datos[fila][3] = 1;  //principio de trazo
					} else {
						datos[fila][3] = 0;  //dentro de trazo
					}
				} else {
					return null;
				}
				//leemos Azi
				if (lineDataTok.hasMoreTokens()) {
					datos[fila][4] = Double.parseDouble(lineDataTok.nextToken());
				} else {
					return null;
				}
				//leemos inc
				if (lineDataTok.hasMoreTokens()) {
					datos[fila][5] = Double.parseDouble(lineDataTok.nextToken());
				} else {
					return null;
				}

				// actualizamos variable fila
				fila++;
			}

		} catch (IOException e) {
			//e.printStackTrace();
			throw new UmbralException("ERROR al abrir el fichero frm" + fichero + ". Pila: " +
					e);
		} finally {

			// Cerramos el fichero pase lo que pase
			if (brFile != null) {
				try {
					brFile.close();
				} catch (IOException e) {
					//e.printStackTrace();
					throw new UmbralException("ERROR al cerrar el fichero ASCII" + fichero + ". Pila: " +
							e);
				}
			}
		}

		return datos;
		
	}

	/**
	 * Lee un fichero en modo ASCII (datos separados por tabuladores)
	 * 
	 * @param fichero
	 * @return
	 * @throws UmbralException 
	 */
	private static double[][] readAsciiFile(String fichero) throws UmbralException {

		// Vemos las lineas del fichero
		int nl = getNumLineasAsciiFile(fichero);

		// Creamos el array de enteros
		double[][] datos = new double[nl][6];

		// Leemos los datos
		try {
			BufferedReader br = new BufferedReader(new FileReader(fichero));
			int fila = 0;
			while (br.ready()) {
				String line = br.readLine();
				if (line != null && line.split("\t").length == 6) {

					String[] cols = line.split("\t");
					for (int i = 0; i < cols.length; i++) {
						String dato = cols[i].replace(".0", "");
						if (dato != null && dato.length() > 0) {
							//datos[fila][i] = Integer.parseInt(dato);
							datos[fila][i] = Double.parseDouble(dato);
						}
					}
					fila++;
				} else {
					break; // Se acabo el fichero...
				}
			}
			br.close();
		} catch (IOException e) {
			//e.printStackTrace();
			throw new UmbralException("ERROR al abrir el fichero ASCII" + fichero + ". Pila: " +
					e);
		}

		// Devolvemos los datos
		return datos;

	}

	/**
	 * Lee los datos del fichero pasado como par�metro y devuelve su contenido
	 * como array de strings
	 * 
	 * @param fichero
	 *            El nombre del fichero de donde leer
	 * @return Array de par�metros de dos dimensiones
	 */
	private static double[][] readBinaryFile(String fichero) {

		// Fichero en binario
		File f = new File(fichero);

		// Suponemos 1 caract.= 4 bytes y que no hay caracteres "superfluos" el
		// el fichero (s�lo est�n las caracter�sticas)
		int numIntegers = (int) (f.length() / 4);
		int numVectors = numIntegers / 6;
		double[][] datosTableta = new double[numVectors][6];
		try {

			// Abrimos el fichero
			DataInputStream dis = new DataInputStream(new FileInputStream(f));

			// Leemos hasta que podamos. Suponemos que el fichero est� bien
			// formado por grupos completos de 5 caracter�sticas almacenadas en
			// datos de tipo entero (enteros de 4 bytes)
			//17-IX-2013: detectado error en este punto, ya que en el resto de formatos el tiempo
			//es el transcurrido entre puntos y no el absoluto como aqu. Lo modifico
			int tAnterior = 0;
			for (int index = 0; index < numVectors; index++) {
				int t = swapBytes(dis.readInt());
				if (index == 0) {
					tAnterior = t;
					t = 0;
				} else {
					int tTemp = t;
					t = t - tAnterior;
					tAnterior = tTemp;
				}
				int x = swapBytes(dis.readInt());
				int y = swapBytes(dis.readInt());
				int p = swapBytes(dis.readInt());
				int a = swapBytes(dis.readInt());
				int i = swapBytes(dis.readInt());
				datosTableta[index] = new double[] { t, x, y, p, a, i };
			}

			// Cerramos el fichero
			dis.close();

		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}

		// Devolvemos los datos le�dos
		return datosTableta;

	}

	/**
	 * Invierte los bytes de un entero
	 * 
	 * @param i
	 * @return
	 */
	private static int swapBytes(int i) {

		int b1 = i & 0xFF000000; // MSB
		int b2 = i & 0x00FF0000;
		int b3 = i & 0x0000FF00;
		int b4 = i & 0x000000FF; // LSB

		int b1_ = b1 >>> 24; // LSB
		int b2_ = b2 >>> 8;
		int b3_ = b3 << 8;
		int b4_ = b4 << 24; // MSB

		return (b1_ + b2_ + b3_ + b4_);
	}

	/**
	 * 
	 * @param theMasterFile
	 * @param userDir
	 * @return
	 * @throws UmbralException 
	 */
	public static ArrayList<String> readMasterFile(String theMasterFile) throws UmbralException {

		// Almacenamos las firmas en un vector
		ArrayList<String> alFirmasIns = null;

		try {

			// Intanciamos el array list
			alFirmasIns = new ArrayList<String>();

			// Abrimos el fichero que contiene las firmas
			BufferedReader brIns = new BufferedReader(new FileReader(
					theMasterFile));

			// Lo recorremos
			while (brIns.ready()) {

				// Leemos el fichero por l�neas
				String line = brIns.readLine();

				// Si la l�nea no contiene datos la saltamos
				if (line == null || line.length() == 0 || line.startsWith("#"))
					continue;

				// A�adimos el nombre del fichero a la lista de plantilas
				alFirmasIns.add(line);

			}

			// Cerramos el fichero de firmas plantilla
			brIns.close();

		} catch (FileNotFoundException e) {
			throw new UmbralException("ERROR al leer el fichero de modelo " + e.toString());
			//System.out.println("ERROR: Fichero " + theMasterFile
			//		+ " no encontrado");
			//e.printStackTrace();
            //            return null;
		} catch (IOException e) {
			throw new UmbralException("ERROR de entrada y salida el leer fichero de modelo " + e.toString());
			//System.out.println(e);
			//e.printStackTrace();
            //            return null;
		}

		// Devolvemos la lista de firmas
		return alFirmasIns;

	}

	/**
	 * 
	 * @param alFirmasIns
	 * @return
	 */
	public static double[][][] readBinaryFiles(ArrayList<String> alFirmasIns) throws UmbralException {

		double[][][] firmas3DArray = new double[alFirmasIns.size()][][];
		for (int i = 0; i < alFirmasIns.size(); i++) {
			double[][] firma2DArray = FileUtil.leeFicheroFirma(alFirmasIns.get(i));
			if (firma2DArray == null) {
				throw new UmbralException("ERROR al leer el fichero de firma " + alFirmasIns.get(i));
			}
			firmas3DArray[i] = firma2DArray;
		}
		return firmas3DArray;
	}
	
	
	/**
	 * 
	 * @param dir
	 * @param filter
	 * @return
	 */
	public static File[] listFiles(String dir, String filter) {

		Filtro filtro = new Filtro(filter);
		File[] files = (new File(dir)).listFiles(filtro);
		return files;

	}

	private static double[][] readTxtFile(String fichero) throws UmbralException {
		// Vemos las lineas del fichero
		int nl = getNumLineasAsciiFile(fichero);

		// Creamos el array de enteros
		double[][] datos = new double[nl][6];

		// Leemos los datos
		try {
			BufferedReader br = new BufferedReader(new FileReader(fichero));
			int fila = 0;
			while (br.ready()) {
				String line = br.readLine();
				if (line != null && line.split(" ").length == 6) {

					String[] cols = line.split(" ");
					for (int i = 0; i < cols.length; i++) {
						if (cols[i] != null && cols[i].length() > 0) {
							datos[fila][i] = Double.parseDouble(cols[i]);
						} else {
							throw new UmbralException("ERROR al leer del fichero txt" + fichero);
							//return null;
						}
					}
					fila++;
				} else {
					break; // Se acabo el fichero...
				}
			}
			br.close();
		} catch (IOException e) {
			//e.printStackTrace();
			throw new UmbralException("ERROR al abrir el fichero txt" + fichero + ". Exception: " + e);
		}

		// Devolvemos los datos
		return datos;
    }
}

/**
 * 
 * @author Juanma
 * 
 */
class Filtro implements FilenameFilter {

	private String filter;

	public Filtro(String filter) {
		this.filter = filter;
	}

	public boolean accept(File dir, String name) {
		if (name.contains(this.filter))
			return true;
		return false;
	}

}
