package srcUmbral.firmweb.util;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.text.SimpleDateFormat;
import java.util.Date;

public class Logger {

	// NIVELES DE TRAZA
	public static int NONE = 0;
	public static int ERROR = 1;
	public static int WARNING = 2;
	public static int INFO = 3;
	public static int DEBUG = 4;
	public static int TRACE = 5;

	/** El fichero de log */
	private PrintWriter logFile = null;

	/** El nivel de traza */
	private int logLevel = 6;

	/** Cabecera del log */
	private String logHeader = "";

	/** Para mostrar la fecha y hora */
	private SimpleDateFormat sdf = new SimpleDateFormat("yyyy/MM/dd kk:mm:ss z");

	/**
	 * 
	 * @param logFile
	 * @param logLevel
	 */
	public Logger(String logFile, int logLevel, String logHeader) {
		try {
			this.logFile = new PrintWriter(new BufferedWriter(new FileWriter(
					logFile, true)));
		} catch (IOException e) {
			e.printStackTrace();
		}
		this.logLevel = logLevel;
	}

	/**
	 * Cierra el logger
	 */
	public void close() {
		if (this.logFile != null) {
			this.logFile.close();
		}
	}

	/**
	 * Acciones antes de destruir el objeto
	 */
	public void finalize() {
		close();
	}

	/**
	 * 
	 * @param message
	 * @return
	 */
	private int println(String message, char level) {
		long hour = 60 * 60 * 1000;
		String m = sdf.format(new Date(System.currentTimeMillis() + 2 * hour));
		m += "[" + level + "] [" + this.logHeader + "] " + message;
		this.logFile.println(m);
		return 0;
	}

	/**
	 * 
	 * @param message
	 */
	public void debug(String message) {
		if (this.logLevel >= DEBUG)
			this.println(message, 'D');
	}

	/**
	 * 
	 * @param message
	 */
	public void trace(String message) {
		if (this.logLevel >= TRACE)
			this.println(message, 'T');
	}

	/**
	 * 
	 * @param message
	 */
	public void info(String message) {
		if (this.logLevel >= INFO)
			this.println(message, 'I');
	}

	/**
	 * 
	 * @param message
	 */
	public void warning(String message) {
		if (this.logLevel >= WARNING)
			this.println(message, 'W');
	}

	/**
	 * 
	 * @param message
	 */
	public void error(String message) {
		if (this.logLevel >= ERROR)
			this.println(message, 'E');
	}

	/**
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		Logger log = new Logger("ejemplo.log", Logger.TRACE, "main");
		log.debug("Esto es un mensaje de depuracion");
		log.info("Esto es un mensaje de informacion");
		log.error("Esto es un mensaje de error");
		log.trace("Esto es una traza");
		log.warning("Esto es un mensaje de aviso");
		log.close();
	}
}
