package srcUmbral.firmweb.util;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Arrays;



public class Maths {

	/**
	 * 
	 * @param arr2D
	 * @return
	 */
	public static double getMean(double[] arr2D) {
		double sum = 0;
		for (int i = 0; i < arr2D.length; i++) {
			sum += arr2D[i];
		}
		return sum / arr2D.length;
	}

	/**
	 * 
	 * @param arr2D
	 * @return
	 */
	public static double getMean(double[] arr2D, double extraValue) {
		double sum = extraValue;
		for (int i = 0; i < arr2D.length; i++) {
			sum += arr2D[i];
		}
		return sum / (arr2D.length + 1);
	}

	/**
	 * 
	 * @param arr2D
	 * @return
	 */
	public static double getMean(double[] arr2D, boolean skipZeros) {
		double sum = 0;
		double numZeros = 0;
		for (int i = 0; i < arr2D.length; i++) {
			sum += arr2D[i];
			if (arr2D[i] == 0)
				numZeros++;
		}
		return sum / (arr2D.length - numZeros);
	}

	/**
	 * 
	 * @param arr2D
	 * @return
	 */
	public static double getStdDev(double[] arr2D) {
		double mean = getMean(arr2D);
		double stdDev = 0;
		for (int i = 0; i < arr2D.length; i++) {
			stdDev += (mean - arr2D[i]) * (mean - arr2D[i]);
		}
		stdDev /= arr2D.length;
		return Math.sqrt(stdDev);
	}

	/**
	 * 
	 * @param arr2D
	 * @return
	 */
	public static double getStdDev(double[] arr2D, double extraValue) {
		double mean = getMean(arr2D, extraValue);
		double stdDev = (mean - extraValue) * (mean - extraValue);
		for (int i = 0; i < arr2D.length; i++) {
			stdDev += (mean - arr2D[i]) * (mean - arr2D[i]);
		}
		stdDev /= (arr2D.length + 1);
		return Math.sqrt(stdDev);
	}

	/**
	 * 
	 * @param arr2D
	 * @return
	 */
	public static double getRelStdDev(double[] arr2D) {
		double mean = getMean(arr2D);
		double stdDev = getStdDev(arr2D);
		return stdDev / mean;
	}

	/**
	 * 
	 * @param array
	 * @return
	 */
	public static double[] getMin(double[] array) {
		double min = array[0];
		int minPos = 0;
		for (int i = 0; i < array.length; i++) {
			if (array[i] < min) {
				min = array[i];
				minPos = i;
			}
		}
		return new double[] { min, minPos };
	}

	/**
	 * 
	 * @param array
	 * @return
	 */
	public static double[] getMax(double[] array) {
		double max = array[0];
		int maxPos = 0;
		for (int i = 0; i < array.length; i++) {
			if (array[i] > max) {
				max = array[i];
				maxPos = i;
			}
		}
		return new double[] { max, maxPos };
	}

	/**
	 * 
	 * @param array
	 * @return
	 */
	public static double[] getMed(double[] array) {
		Arrays.sort(array);
		double med = array[array.length / 2 + 1];
		return new double[] { med, array.length / 2 + 1 };
	}

	/**
	 * 
	 * @param array
	 * @return
	 */
	public static double[] getMad(double[] array) {
		double med = getMed(array)[0];
		double[] medArray = new double[array.length];
		for (int i = 0; i < array.length; i++) {
			medArray[i] = Math.abs(array[i] - med);
		}
		double[] mad = getMed(medArray);
		return mad;
	}

	/**
	 * Calcula la primera derivada temporal de la matriz de caractersticas
	 * 
	 * @param xytpai
	 *            La matriz NxM de caractersticas a derivar
	 * @return La matriz (N-1)xM con la derivada calculada como C(i+1) - C(i)
	 */
	public static double[][] derivate(double[][] xytpai) {
		final int T = 2;
		double[][] ddata = new double[xytpai.length - 1][];
		for (int i = 0; i < ddata.length; i++) {

			// Inicializamos el array con valores grandes
			double[] d_xytpai = ArraysUtil.createArray(xytpai[i].length,
					Double.MAX_VALUE);

			// Slo almacenamos si no hay saltos de posicin (de ms de 20 ms)
			if (xytpai[i + 1][T] < 20 || 1 == Math.random()) {
				d_xytpai = derivate(xytpai[i], xytpai[i + 1]);
			}

			// Almacenamos en la matriz de salida
			ddata[i] = d_xytpai;

		}
		return ddata;
	}

	/**
	 * Resta dos vectores exceptuando la columna del tiempo (la 2)
	 * 
	 * @param xytpai_n
	 * @param xytpai_nn
	 * @return
	 */
	public static double[] derivate(double[] xytpai_n, double[] xytpai_nn) {
		if (xytpai_n == null || xytpai_nn == null
				|| xytpai_n.length != xytpai_nn.length) {
			return null;
		}
		final int T = 2;
		double[] d_xytpai = new double[xytpai_n.length];
		for (int i = 0; i < d_xytpai.length; i++) {

			// La columna de tiempo no la derivamos
			if (i == T) {
				d_xytpai[i] = xytpai_n[i];
				continue;
			}

			// El resto de columnas se derivan sin ms
			d_xytpai[i] = xytpai_nn[i] - xytpai_n[i];
			if (xytpai_nn[T] != 0) {
				d_xytpai[i] /= xytpai_nn[T];
			}
		}
		return d_xytpai;
	}

	/**
	 * Calcula la primera derivada temporal
	 * 
	 * @param xytpai
	 * @param onlyVisible
	 *            Si true salta los segmentos sin presin
	 * @return
	 */
	public static double[][] derivate(double[][] xytpai, boolean onlyVisible) {

		// Si procesamos los segmentos con y sib sin tinta
		if (onlyVisible == false) {
			return derivate(xytpai);
		}

		// Si saltamos los segmentos sin presin
		final int PRESION_POS = 3;
		final int TIME_POS = 2;
		int nc = xytpai[0].length;
		ArrayList<double[]> dDataList = new ArrayList<double[]>();
		for (int fil = 0; fil < xytpai.length - 1; fil++) {

			// Slo derivamos entre dos puntos seguidos con presin
			if (xytpai[fil][PRESION_POS] > 0
					&& xytpai[fil + 1][PRESION_POS] > 0) {

				// Derivamos todas las columnas menos la del tiempo
				double[] derivate = new double[nc];
				for (int col = 0; col < nc; col++) {
					if (col == TIME_POS) {
						derivate[col] = xytpai[fil + 1][col];
						continue;
					}
					derivate[col] = xytpai[fil + 1][col] - xytpai[fil][col];
					if (xytpai[fil + 1][TIME_POS] > 0) {
						derivate[col] /= xytpai[fil + 1][TIME_POS];
					}
				}
				dDataList.add(derivate);
			}
		}

		// Pasamos al array los datos de la lista
		double[][] dData = new double[dDataList.size()][nc];
		for (int i = 0; i < dDataList.size(); i++) {
			dData[i] = dDataList.get(i);
		}
		return dData;
	}

	/**
	 * Calcula la segunda derivada temporal
	 * 
	 * @param xytpai
	 * @param onlyVisible
	 *            Si true salta los segmentos sin presin
	 * @return
	 */
	public static double[][] dderivate(double[][] xytpai, boolean onlyVisible) {

		// Si derivamos todos los puntos
		if (onlyVisible == false) {
			return derivate(derivate(xytpai));
		}

		// Si saltamos los segmentos sin presin
		final int PRESION_POS = 3;
		final int TIME_POS = 2;
		int nc = xytpai[0].length;
		ArrayList<double[]> ddDataList = new ArrayList<double[]>();
		for (int fil = 0; fil < xytpai.length - 2; fil++) {

			// Slo derivamos entre tres puntos seguidos con presin
			if (xytpai[fil][PRESION_POS] > 0
					&& xytpai[fil + 1][PRESION_POS] > 0
					&& xytpai[fil + 2][PRESION_POS] > 0) {

				// Derivamos todas las columnas del array menos el tiempo
				double[] derivate = new double[nc];
				for (int col = 0; col < nc; col++) {

					if (col == TIME_POS) {
						derivate[col] = xytpai[fil + 1][col];
						continue;
					}

					double dData0 = xytpai[fil + 1][col] - xytpai[fil + 0][col];
					dData0 /= xytpai[fil + 1][TIME_POS];

					double dData1 = xytpai[fil + 2][col] - xytpai[fil + 1][col];
					dData1 /= xytpai[fil + 2][TIME_POS];

					derivate[col] = dData1 - dData0;
					double dt = xytpai[fil + 1][TIME_POS]
							+ xytpai[fil + 2][TIME_POS];
					if (dt > 0) {
						derivate[col] /= dt;
					}

				}
				ddDataList.add(derivate);
			}
		}

		// Pasamos a array la lista
		double[][] ddData = new double[ddDataList.size()][nc];
		for (int i = 0; i < ddDataList.size(); i++) {
			ddData[i] = ddDataList.get(i);
		}
		return ddData;
	}

	/**
	 * 
	 * @param xypai
	 * @return
	 */
	public static int numInvisibleSegments(double[][] xypai) {
		int numInvSegments = 0; // Number of Invisible segments
		boolean startInvSeg = false;
		final int PRESION_POS = 3;
		for (int i = 0; i < xypai.length; i++) {
			if (xypai[i][PRESION_POS] == 0 && startInvSeg == false) {
				startInvSeg = true;
				numInvSegments++;
			} else if (xypai[i][PRESION_POS] != 0 && startInvSeg == true) {
				startInvSeg = false;
			}
		}
		return numInvSegments;
	}

	/**
	 * 
	 * @param x0
	 * @param y0
	 * @param x1
	 * @param y1
	 * @return
	 */
	public static double euclDist(double x0, double y0, double x1, double y1) {
		double dx = x0 - x1;
		double dy = y0 - y1;
		double dist = Math.sqrt(dx * dx + dy * dy);
		return dist;
	}

	/**
	 * 
	 * @param intraDists
	 * @param interDists
	 * @return
	 */
	public static double[] getMaxRatio(double[] intraDists, double[] interDists) {
		double[] interIntraRatio = new double[intraDists.length];
		int maxRatioPos = 0;
		double maxRatioVal = interDists[maxRatioPos] / intraDists[maxRatioPos];
		for (int i = 0; i < interIntraRatio.length; i++) {
			double ratioVal = interDists[i] / intraDists[i];
			if (maxRatioVal < ratioVal) {
				maxRatioVal = ratioVal;
				maxRatioPos = i;
			}
		}
		return new double[] { maxRatioVal, maxRatioPos };
	}

	/**
	 * 
	 * @param number
	 * @param decimals
	 * @return
	 */
	public static String round(double number, int decimals) {
		String s = "";
		for (int i = 0; i < decimals; i++) {
			if (i == 0)
				s += ".";
			s += "0";
		}
		DecimalFormat df = new DecimalFormat("0" + s);
		return df.format(number).replace(',', '.');
	}

	/**
	 * 
	 * @param doubles
	 * @param decimals
	 * @return
	 */
	public static double[] round(double[] doubles, int decimals) {
		double[] arr = new double[doubles.length];
		for (int i = 0; i < arr.length; i++) {
			arr[i] = Double.parseDouble(round(doubles[i], decimals));
		}
		return arr;
	}
	
	  /**
	   * 
	   * @param scores
	   * @param perc
	   * @return
	   * 
	   *   Calculo:
	   *   		*Si i=(num_datos*per)/100 es exacto: elemento(i)+elemento(i+1)/2
	   *   	    *Si i=(num_datos*per)/100 no es exacto: elemento(i+1)
	   *   IMPORTANTE:
	   *   	    *El calculo de indice se hace como si el primero fuera 1, pero como es cero,
	   *     en realidad al indice calculado habra que restarle uno para que coincidiera
	   *     con el del calculo anterior. Por eso los indices del codigo son una unidad menor
	   *     a los mostrados arriba.
	   */
	  public static double percentil(double[] scores, int perc) {
		  int num_datos = scores.length;
		  double[] scores_ord = scores; 
		  
		  Arrays.sort(scores_ord);
		  /** Comprobacion **
		  for (int i=0; i<num_datos; i++) {
			  System.out.print(scores_ord[i] + " ");
		  }
		  System.out.println();
		  ******************/
		  if ( ((double) (num_datos*perc)%100.0) == 0) {
			  int indice = (int) ((double) (num_datos*perc)/100.0);
			  return (scores_ord[indice-1]+scores_ord[indice]) / 2.0;  
		  } else {
			  int indice = (int) Math.floor((double) (num_datos*perc)/100.0);
			  //System.out.println("Indice: " + indice);
			  return scores_ord[indice];
		  }
		  
		  
	  }


}
