package srcUmbral.umbral;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Formatter;

import srcUmbral.MisExcepciones.UmbralException;

import srcUmbral.configuracion.Configuracion;
import srcUmbral.firmweb.util.ArraysUtil;

public class Eer {

	public double eerValue;
	public double eerThreshold;
	public double farValue;
	public double frrValue;
	public double eerLowerScore;
	public double eerUpperScore;
	public double highestGenuineScore;
	public double lowestForgeryScore;
	public double bestRejectedForgery;
	public double worstAcceptedGenuine;
	public double scoresGap;
	//02-11-2011 aadidos estos valores para umbrales de Frr y Far
	public double farThreshold;
	public double frrThreshold;
	  
	// Variables para el calculo del corte en el nuevo calculo del EER
	  
	private static double umbral_tmp;
	private static double EER_tmp;

	/**
	 * 
	 * @param scores
	 * @return
	 */
	public static double getLowestForgeryScore(Score[] scores) {
		double ret = Double.MAX_VALUE;
		for (int i = 0; i < scores.length; i++) {
			Score s = scores[i];
			if (!s.isAutentica() && s.getValor() < ret) {
				ret = s.getValor();
			}
		}
		return ret;
	}

	/**
	 * 
	 * @param scores
	 * @return
	 */
	public static double getHighestGenuineScore(Score[] scores) {
		double ret = -Double.MAX_VALUE;
		for (int i = 0; i < scores.length; i++) {
			Score s = scores[i];
			if (s.isAutentica() && s.getValor() > ret) {
				ret = s.getValor();
			}
		}
		return ret;
	}

	/**
	 * Devuelve la tasa de falsas aceptaciones
	 * 
	 * @param umbral
	 * @param scores
	 * @return
	 */
	public static double getFar(double umbral, Score[] scores) {
		double numImitaciones = 0;
		for (int i = 0; i < scores.length; i++) {
			if (scores[i].isAutentica() == false)
				numImitaciones++;
		}
		int fa = 0;
		for (int i = 0; i < scores.length; i++) {
			if (scores[i].getValor() < umbral
					&& scores[i].isAutentica() == false)
				fa++;
		}
		double far = fa / numImitaciones;
		return far;
	}

	/**
	 * Devuelve la tasa de falsas aceptaciones
	 * 
	 * @param umbral
	 * @param scores
	 * @return
	 */
	public static double getFar(double umbral, double[] impostors) {
		int fa = 0;
		for (int i = 0; i < impostors.length; i++) {
			if (impostors[i] < umbral)
				fa++;
		}
		double far = fa / (double) impostors.length;
		return far;
	}

	/**
	 * Devuelve la tasa de falsos rechazos
	 * 
	 * @param umbral
	 * @param scores
	 * @return
	 */
	public static double getFrr(double umbral, Score[] scores) {
		double numVerdaderasParaProbar = 0;
		for (int i = 0; i < scores.length; i++) {
			if (scores[i].isAutentica() == true)
				numVerdaderasParaProbar++;
		}
		int contadorFr = 0;
		for (int i = 0; i < scores.length; i++) {
			if (scores[i].getValor() > umbral
					&& scores[i].isAutentica() == true)
				contadorFr++;
		}
		double frr = contadorFr / numVerdaderasParaProbar;
		return frr;
	}

	/**
	 * Devuelve la tasa de falsos rechazos
	 * 
	 * @param umbral
	 * @param scores
	 * @return
	 */
	public static double getFrr(double umbral, double[] genuines) {
		int contadorFr = 0;
		for (int i = 0; i < genuines.length; i++) {
			if (genuines[i] > umbral)
				contadorFr++;
		}
		double frr = contadorFr / (double) genuines.length;
		return frr;
	}

	/**
	 * Calcula la tasa de falsas identificaciones
	 * 
	 * @param usScores
	 * @param cliIndexes
	 * @return
	 */
	public static double getFir(double[][] usScores, int[] cliIndexes) {
		double idErrs = 0;
		for (int us = 0; us < usScores.length; us++) {
			idErrs += idError(usScores[us], cliIndexes[us]);
		}
		double fir = 100 * (idErrs / usScores.length);
		return fir;
	}

	/**
	 * 
	 * @param scoCliUsr
	 * @return
	 */
	private static int idError(double[] idScores, int cliIndex) {
		int minIndex = 0;
		double minScore = idScores[minIndex];
		for (int i = 0; i < idScores.length; i++) {
			if (idScores[i] < minScore) {
				minIndex = i;
				minScore = idScores[i];
			}
		}
		return (minIndex == cliIndex) ? 0 : 1;
	}

	/**
	 * Calcula la tasa de equi-error
	 * 
	 * @param scores
	 * @return
	 */
	// public static double calcEer1(Score[] scores)
	// {
	// Score[] scoresCopy = new Score[scores.length];
	// for (int i = 0; i < scoresCopy.length; i++) {
	// scoresCopy[i] = scores[i];
	// }
	// Arrays.sort(scoresCopy);
	//
	// boolean comparacion = false;
	// int i = 0;
	// double umbral = 0;
	// for (i = 0; i < scoresCopy.length - 1; i++) {
	// double scoreAntes = scoresCopy[i].getValor();
	// double scoreDespues = scoresCopy[i + 1].getValor();
	// umbral = (scoreAntes + scoreDespues) / 2;
	// double far = getFar(umbral, scoresCopy);
	// double frr = getFrr(umbral, scoresCopy);
	// if (i == 0) {
	// comparacion = (far > frr);
	// }
	// if ((far > frr) != comparacion) {
	// break;
	// }
	// }
	// double worstGenuine = getWorstGenuine(scoresCopy);
	// double bestForgery = getBestForgery(scoresCopy);
	// double umbralFar = (scoresCopy[i - 1].getValor() +
	// scoresCopy[i].getValor()) / 2;
	// double umbralFrr = (scoresCopy[i].getValor() + scoresCopy[i +
	// 1].getValor()) / 2;
	// double far = getFar(umbralFar, scoresCopy) * 100;
	// double frr = getFrr(umbralFrr, scoresCopy) * 100;
	// double eer = (far + frr) / 2;
	//
	// if (1 > Math.random()) {
	// DecimalFormat df = new DecimalFormat("0.00");
	// System.out.print("EER=" + df.format(eer) + "\t");
	// System.out.print("W_GEN=" + worstGenuine + "\t");
	// System.out.print("B_FOR=" + bestForgery + "\t");
	// System.out.println();
	// }
	// return eer;
	// }

	/**
	 * Calcula EER con umbral universal
	 * 
	 * @param scoCli
	 * @param scoFor
	 * @return
	 * @throws UmbralException 
	 */
	public static Eer getEer(double[][] scoCliUsers, double[][] scoForUsers) throws UmbralException {
		double[] scoCli = ArraysUtil.merge(scoCliUsers);
		double[] scoFor = ArraysUtil.merge(scoForUsers);
		return getEer(scoCli, scoFor);
	}

	/**
	 * 
	 * @param scores
	 * @return
	 */
	public static Eer getEer(Score[] scores) {
		Score[] scoresCopy = new Score[scores.length];
		for (int i = 0; i < scoresCopy.length; i++) {
			scoresCopy[i] = scores[i];
		}
		Arrays.sort(scoresCopy);

		boolean comparacion = false;
		int i = 0;
		double umbral = 0;
		for (i = 0; i < scoresCopy.length - 1; i++) {
			double scoreAntes = scoresCopy[i].getValor();
			double scoreDespues = scoresCopy[i + 1].getValor();
			umbral = (scoreAntes + scoreDespues) / 2;
			double far = getFar(umbral, scoresCopy);
			double frr = getFrr(umbral, scoresCopy);
			if (i == 0) {
				comparacion = (far > frr);
			}
			if ((far > frr) != comparacion) {
				break;
			}
		}
		if (i >= scoresCopy.length - 1)
			throw new RuntimeException("No se puede calcular el EER");
		double umbralFar = (scoresCopy[i - 1].getValor() + scoresCopy[i]
				.getValor()) / 2;
		double umbralFrr = (scoresCopy[i].getValor() + scoresCopy[i + 1]
				.getValor()) / 2;
		double far = getFar(umbralFar, scoresCopy) * 100;
		double frr = getFrr(umbralFrr, scoresCopy) * 100;
		double eer = (far + frr) / 2;

		Eer eerObj = new Eer();
		eerObj.eerValue = eer;
		eerObj.eerThreshold = (umbralFar + umbralFrr) / 2;
		eerObj.farValue = far;
		eerObj.frrValue = frr;
		eerObj.eerLowerScore = scoresCopy[i].getValor();
		eerObj.eerUpperScore = scoresCopy[i + 1].getValor();
		eerObj.bestRejectedForgery = getBestRejectedForgery(scoresCopy,
				eerObj.eerThreshold);
		eerObj.worstAcceptedGenuine = getWorstAcceptedGenuine(scoresCopy,
				eerObj.eerThreshold);
		eerObj.lowestForgeryScore = getLowestForgeryScore(scoresCopy);
		eerObj.highestGenuineScore = getHighestGenuineScore(scoresCopy);
		eerObj.scoresGap = (eerObj.lowestForgeryScore - eerObj.highestGenuineScore)
				/ (eerObj.lowestForgeryScore + eerObj.highestGenuineScore);

		if (1 == Math.random()) {
			DecimalFormat df = new DecimalFormat("0.00");
			System.out.print("EER=" + df.format(eer) + "\t");
			System.out.print("W_GEN=" + eerObj.eerLowerScore + "\t");
			System.out.print("B_FOR=" + eerObj.eerUpperScore + "\t");
			System.out.println();
		}

		return eerObj;
	}
	
	/**
	   * Modificacion (27-10-2011) modifico yo
	   * 
	   * @param scores
	   * @return
	 * @throws UmbralException 
	   */
	  public static Eer getEer2(Score[] scores) throws UmbralException
	  {
	    Score[] scoresCopy = new Score[scores.length];
//	    double [] punto_corte = new double[2];  // almacenamos umbral y eer
	    
	    for (int i = 0; i < scoresCopy.length; i++) {
	      scoresCopy[i] = scores[i];
	    }
	    Arrays.sort(scoresCopy);
	    
	    if (Configuracion.aplicar_media_si_EER_0) {
	    	double max_cli = 0.0;
	    	double min_imp = 0.0;
	    	for (int i=0; i<scoresCopy.length; i++) {
	    		if (scoresCopy[i].isAutentica()) {
	    			max_cli = scoresCopy[i].getValor(); // Al acabar tiene el maximo
	    		}
	    	}
	    	int i=0;
	    	while (scoresCopy[i].isAutentica()) {
	    		i++;
	    	}
	    	min_imp = scoresCopy[i].getValor();
	    	
	    	if (max_cli == 0.0 || min_imp == 0.0) {
	    		throw new UmbralException("ERROR en calculo de EER: al calcular max_cli y min_imp");
	    		//System.out.println("ERROR al calcular max_cli y min_imp");
	    		//System.exit(1);
	    	}
	    	
	    	if (max_cli <= min_imp) {
	    		double eer = 0.0;
	    	    double umbral = (max_cli+min_imp)/2.0;
	    	    Eer eerObj = new Eer();
	    	    eerObj.eerValue = eer;
	    	    eerObj.eerThreshold = umbral;
	    	    eerObj.farValue = getFar(umbral, scoresCopy) * 100;
	    	    eerObj.frrValue = getFrr(umbral, scoresCopy) * 100;
	    	    eerObj.eerLowerScore = max_cli;
	    	    eerObj.eerUpperScore = min_imp;
	    	    eerObj.bestRejectedForgery = getBestRejectedForgery(scoresCopy,
	    	        eerObj.eerThreshold);
	    	    eerObj.worstAcceptedGenuine = getWorstAcceptedGenuine(scoresCopy,
	    	        eerObj.eerThreshold);
	    	    return eerObj;
	    	}
	    	
	    }

	    // Calculo del primer punto de corte
	    boolean comparacion = false;
	    int i = 0;
	    double pto_calculo;
	    for (i = 0; i < scoresCopy.length - 1; i++) {
//	      double scoreAntes = scoresCopy[i].getValor();
//	      double scoreDespues = scoresCopy[i + 1].getValor();
	      pto_calculo = scoresCopy[i].getValor();
	      double far = getFar(pto_calculo, scoresCopy);
	      double frr = getFrr(pto_calculo, scoresCopy);

	      if (i == 0) {
	        comparacion = (far > frr);
	      }
	      if ((far > frr) != comparacion) {
	        break;
	      }
	    }
	    if (i >= scoresCopy.length - 1)
	      throw new RuntimeException("No se puede calcular el EER");
	    
	    double umbral_s = scoresCopy[i].getValor();
	    double umbral_i = scoresCopy[i-1].getValor();
	    double far_i = getFar(umbral_i, scoresCopy) * 100;
	    double frr_i = getFrr(umbral_i, scoresCopy) * 100;
	    double far_s = getFar(umbral_s, scoresCopy) * 100;
	    double frr_s = getFrr(umbral_s, scoresCopy) * 100; 
	    calcula_corte(far_i,frr_i,umbral_i,far_s,frr_s,umbral_s);  // calculamos umbral y EER provisionales
	    int pto_corte_inf = i-1;  // Los necesito para param eerLower y eerUpper Scores
	    int pto_corte_sup = i;
	    
	    double paso = (umbral_s - umbral_i) / 10;   // afino el intervalo de corte
	    int num_it =1;
	    double EER_ant;
	     do {
	    	 for (i=0; i<=10; i++) {
	    		 pto_calculo = umbral_i + (i*paso);
	    		 double far = getFar(pto_calculo, scoresCopy);
	    	     double frr = getFrr(pto_calculo, scoresCopy);
	    	     if (i == 0) {
	    	         comparacion = (far > frr);
	    	       }
	    	       if ((far > frr) != comparacion) {
	    	         break;
	    	       }
	    	 }
	    	 umbral_s = umbral_i + (i*paso);
	    	 umbral_i = umbral_i + ((i-1)*paso);
	    	 far_i = getFar(umbral_i, scoresCopy) * 100;
	    	 frr_i = getFrr(umbral_i, scoresCopy) * 100;
	    	 far_s = getFar(umbral_s, scoresCopy) * 100;
	    	 frr_s = getFrr(umbral_s, scoresCopy) * 100;
	    	 EER_ant = EER_tmp;
	    	 calcula_corte(far_i,frr_i,umbral_i,far_s,frr_s,umbral_s);  // calculamos umbral y EER provisionales
	    	 num_it++;
	    	 if (num_it >100) {   // 100 iteraciones maximio
	    		 throw new UmbralException("Excedido el numero de iteraciones");
	    	 }
	    	 paso = (umbral_s - umbral_i) / 10;   // afino el intervalo de corte
	     } while(Math.abs(EER_tmp - EER_ant) > 0.01);
	    

	    double eer = EER_tmp;
	    double umbral = umbral_tmp;
	    Eer eerObj = new Eer();
	    eerObj.eerValue = eer;
	    eerObj.eerThreshold = umbral;
	    eerObj.farValue = getFar(umbral, scoresCopy) * 100;
	    eerObj.frrValue = getFrr(umbral, scoresCopy) * 100;
	    eerObj.eerLowerScore = scoresCopy[pto_corte_inf].getValor();
	    eerObj.eerUpperScore = scoresCopy[pto_corte_sup].getValor();
	    eerObj.bestRejectedForgery = getBestRejectedForgery(scoresCopy,
	        eerObj.eerThreshold);
	    eerObj.worstAcceptedGenuine = getWorstAcceptedGenuine(scoresCopy,
	        eerObj.eerThreshold);

	    if (1 == 0) {
	      DecimalFormat df = new DecimalFormat("0.00");
	      System.out.print("EER=" + df.format(eer) + "\t");
	      System.out.print("W_GEN=" + eerObj.eerLowerScore + "\t");
	      System.out.print("B_FOR=" + eerObj.eerUpperScore + "\t");
	      System.out.println();
	    }

	    return eerObj;
	  }
	  
	  private static void calcula_corte(double fa_i, double fr_i, double umbral_i, double fa_s, double fr_s, double umbral_s) {
		  
		  double pendiente_1, ord_orige_1,pendiente_2, ord_orige_2;
		  
		  /*** calculo de la pendiente y ordenada en origen de la recta que baja (fr_i -> fr_s) ***/
		  pendiente_1 = (fr_i - fr_s) / (umbral_i - umbral_s);
		  ord_orige_1 = (umbral_i*fr_s - umbral_s*fr_i) / (umbral_i - umbral_s);
		  
		  /*** calculo de la pendiente y ordenada en origen de la recta que sube (fa_i -> fa_s) ***/
		  pendiente_2 = (fa_i - fa_s) / (umbral_i - umbral_s);
		  ord_orige_2 = (umbral_i*fa_s - umbral_s*fa_i) / (umbral_i - umbral_s);
		  
		  /*** Calculo punto de corte ***/
		  //umbral_tmp =  (ord_orige_2 - ord_orige_1) / (pendiente_1 - pendiente_2);  /** abscisa **/
		  //EER_tmp = (pendiente_1*ord_orige_2 - pendiente_2*ord_orige_1) / (pendiente_1 - pendiente_2);  /** ordenada **/
		  umbral_tmp =  (ord_orige_1 - ord_orige_2) / (pendiente_2 - pendiente_1);  /** abscisa **/
		  EER_tmp = Math.abs((pendiente_2*ord_orige_1 - pendiente_1*ord_orige_2) / (pendiente_2 - pendiente_1));  /** ordenada **/
	  }

	/**
	 * 
	 * @param scores
	 * @param eerThreshold
	 * @return
	 */
	public static double getWorstAcceptedGenuine(Score[] scores,
			double eerThreshold) {
		Score[] scoresCopy = Score.clone(scores);
		Arrays.sort(scoresCopy);
		double wag = scoresCopy[0].getValor();
		for (int i = 0; i < scoresCopy.length; i++) {
			if (scoresCopy[i].getValor() > eerThreshold)
				break;
			if (scoresCopy[i].isAutentica()) {
				wag = scoresCopy[i].getValor();
			}
		}
		return wag;
	}

	/**
	 * 
	 * @param scores
	 * @param eerThreshold
	 * @return
	 */
	public static double getBestRejectedForgery(Score[] scores,
			double eerThreshold) {
		Score[] scoresCopy = Score.clone(scores);
		Arrays.sort(scoresCopy);
		double brf = scoresCopy[0].getValor();
		for (int i = 0; i < scoresCopy.length; i++) {
			if (scoresCopy[i].getValor() < eerThreshold)
				continue;
			if (!scoresCopy[i].isAutentica()) {
				brf = scoresCopy[i].getValor();
				break;
			}
		}
		return brf;
	}

	/**
	 * Calcula la tasa de equierror
	 * 
	 * @param scores
	 * @return
	 * @throws UmbralException 
	 */
	public static Eer getEer(double[] genuineScores, double[] impostorScores) throws UmbralException {
		// Almacenamos los datos en un array de scores
		Score[] scores = new Score[genuineScores.length + impostorScores.length];
		for (int i = 0; i < genuineScores.length; i++) {
			scores[i] = new Score(genuineScores[i], true);
		}
		for (int i = genuineScores.length; i < scores.length; i++) {
			scores[i] = new Score(impostorScores[i - genuineScores.length],
					false);
		}

		// Llamamos a la funcin principal
		//27-01-2012 Cambio por mi calculo
//Orig.		Eer eer = getEer(scores);
		Eer eer = new Eer();
		eer = getEer2(scores);
	    
		//Eer eer = getEer2(scores);
		return eer;
	}
	
	  /**
	   * Calcula el umbral mas alto y la Far para ese umbral para un valor fijo de Frr
	   * Esta funcion solo funciona con scores tipo distancia, es decir,
	   *   fijado un umbral si el scores es menor decimos que pertenece al cliente
	   *   y si es mayor a impostor.
	   * El sentido de la comparacion es diferente en el caso de salida tipo probabilidad.
	   * 
	   * En realidad con las salidas del cliente valdra, pero al incluir las de impostores
	   *   tengo mas datos por si les necesito.
	   * 
	   * @param genuineScores
	   * @param impostorScores
	   * @param Frr
	   * @return
	 * @throws UmbralException 
	   */
	  public static Eer Frr_fixed(double[] genuineScores, double[] impostorScores, 
			  double Frr_fix) throws UmbralException {
		  
			// Almacenamos los datos en un array de scores
			Score[] scores = new Score[genuineScores.length + impostorScores.length];
			for (int i = 0; i < genuineScores.length; i++) {
				scores[i] = new Score(genuineScores[i], true);
			}
			for (int i = genuineScores.length; i < scores.length; i++) {
				scores[i] = new Score(impostorScores[i - genuineScores.length],
						false);
			}
			
			// Ordeno los scores para recorrerlo usandolos como umbral
			Arrays.sort(scores);
			
			int i = 0;
			double Frr = getFrr(scores[i].getValor(), scores); //Valor inicial
			while(Frr > Frr_fix) {
				i++;
				Frr = getFrr(scores[i].getValor(), scores);
			}
			
			// Calculado punto de corte rellenamos el objeto Score
			
			double umbral;
			if (i!=(scores.length-1))
				umbral = (scores[i].getValor() + scores[i+1].getValor()) / 2.0; //Fijo este valor como umbral
			else
				umbral = scores[i].getValor();
		    Eer eerObj = new Eer(); //Lo que retornaremos
		    Eer eer = new Eer();  // Para almacenar parte de calculo de EER
		    
		    // Calculo el EER para tenerlo y poderlo aadir el fichero weka
		    eer = getEer2(scores);
		    eerObj.eerValue = eer.eerValue;
		    eerObj.eerThreshold = eer.eerThreshold;
		    eerObj.farValue = getFar(umbral, scores) * 100;
		    eerObj.farThreshold = umbral;
		    eerObj.frrValue = getFrr(umbral, scores) * 100;  //Debe ser el buscado. Lo dejo por comprobacion
		    eerObj.frrThreshold = umbral;
		    eerObj.eerLowerScore = eer.eerLowerScore; //Dejamos este de momento
		    eerObj.eerUpperScore = eer.eerUpperScore; //Dejamos este de momento
		    eerObj.bestRejectedForgery = getBestRejectedForgery(scores,
		        eerObj.eerThreshold);
		    eerObj.worstAcceptedGenuine = getWorstAcceptedGenuine(scores,
		        eerObj.eerThreshold);

			return eerObj;
	  }
	  
	  /**
	   * Calcula el umbral mas alto y la Far para ese umbral para un valor fijo de Frr
	   * Esta funcion solo funciona con scores tipo distancia, es decir,
	   *   fijado un umbral si el scores es menor decimos que pertenece al cliente
	   *   y si es mayor a impostor.
	   * El sentido de la comparacion es diferente en el caso de salida tipo probabilidad.
	   * 
	   * En realidad con las salidas del cliente valdra, pero al incluir las de impostores
	   *   tengo mas datos por si les necesito.
	   * 
	   * @param genuineScores
	   * @param impostorScores
	   * @param Frr
	   * @return
	 * @throws UmbralException 
	   */
	  public static Eer Far_fixed(double[] genuineScores, double[] impostorScores, 
			  double Far_fix) throws UmbralException {
		  
			// Almacenamos los datos en un array de scores
			Score[] scores = new Score[genuineScores.length + impostorScores.length];
			for (int i = 0; i < genuineScores.length; i++) {
				scores[i] = new Score(genuineScores[i], true);
			}
			for (int i = genuineScores.length; i < scores.length; i++) {
				scores[i] = new Score(impostorScores[i - genuineScores.length],
						false);
			}
			
			// Ordeno los scores para recorrerlo usandolos como umbral
			Arrays.sort(scores);
			
			int i = scores.length-1;
			double Far = getFar(scores[i].getValor(), scores); //Valor inicial
			while(Far > Far_fix) {
				i--;
				Far = getFar(scores[i].getValor(), scores);
			}
			
			// Calculado punto de corte rellenamos el objeto Score
			
			double umbral;
			if (i!=0)
				umbral = (scores[i].getValor() + scores[i-1].getValor()) / 2.0; //Fijo este valor como umbral
			else
				umbral = scores[i].getValor();
		    Eer eerObj = new Eer(); //Lo que retornaremos
		    Eer eer = new Eer();  // Para almacenar parte de calculo de EER
		    
		    // Calculo el EER para tenerlo y poderlo aadir el fichero weka
		    eer = getEer2(scores);
		    eerObj.eerValue = eer.eerValue;
		    eerObj.eerThreshold = eer.eerThreshold;
		    eerObj.farValue = getFar(umbral, scores) * 100;  //Debe ser el buscado. Lo dejo por comprobacion
		    eerObj.farThreshold = umbral;
		    eerObj.frrValue = getFrr(umbral, scores) * 100;
		    eerObj.frrThreshold = umbral;
		    eerObj.eerLowerScore = eer.eerLowerScore; //Dejamos este de momento
		    eerObj.eerUpperScore = eer.eerUpperScore; //Dejamos este de momento
		    eerObj.bestRejectedForgery = getBestRejectedForgery(scores,
		        eerObj.eerThreshold);
		    eerObj.worstAcceptedGenuine = getWorstAcceptedGenuine(scores,
		        eerObj.eerThreshold);

			return eerObj;
	  }

	/**
	 * 
	 * @param genuinesFile
	 * @param forgeriesFile
	 * @return
	 * @throws UmbralException 
	 */
	public static Eer getEer(String genuinesFile, String forgeriesFile) throws UmbralException {
		double[] genuineScores = readScoresFile(genuinesFile);
		double[] forgeriesScores = readScoresFile(forgeriesFile);
		return getEer(genuineScores, forgeriesScores);
	}

	/**
	 * 
	 * @param genuinesFile
	 * @param forgeriesFile
	 * @return
	 * @throws UmbralException 
	 */
	public static Eer getEer(String[] genuinesFiles, String[] forgeriesFiles) throws UmbralException {
		ArrayList<Double> genuinesScoresList = new ArrayList<Double>();
		for (int i = 0; i < genuinesFiles.length; i++) {
			double[] genuineScores = readScoresFile(genuinesFiles[i]);
			for (int j = 0; j < genuineScores.length; j++) {
				genuinesScoresList.add(genuineScores[j]);
			}
		}
		ArrayList<Double> forgeriesScoresList = new ArrayList<Double>();
		for (int i = 0; i < forgeriesFiles.length; i++) {
			double[] forgeriesScores = readScoresFile(forgeriesFiles[i]);
			for (int j = 0; j < forgeriesScores.length; j++) {
				forgeriesScoresList.add(forgeriesScores[j]);
			}
		}
		double[] genuinesScoresAll = ArraysUtil
				.toDoubleArray(genuinesScoresList);
		double[] forgeriesScoresAll = ArraysUtil
				.toDoubleArray(forgeriesScoresList);
		return getEer(genuinesScoresAll, forgeriesScoresAll);
	}

	
	/**
	 * Lee un fichero de scores y lo devuelve en un array
	 * 
	 * @param file
	 * @return
	 */
	public static double[] readScoresFile(String file) {
		ArrayList<Double> list = new ArrayList<Double>();
		try {
			BufferedReader br = new BufferedReader(new FileReader(file));
			while (br.ready()) {
				String line = br.readLine();
				if (line == null || line.trim().length() == 0) {
					continue;
				}
				line = line.replaceAll(",", ".");
				try {
					double score = Double.parseDouble(line);
					list.add(score);
				} catch (NumberFormatException e) {
					e.printStackTrace();
				}
			}
			br.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return ArraysUtil.toDoubleArray(list);
	}

	/**
	 * Mtodo principal
	 * 
	 * @param args
	 */
	/*
	public static void main(String[] args) {
		if (1 > Math.random()) {
			String dir = "dist/submission/task1-ds2/documents";
			String clientS1File = dir + "/batchMatchingClientS1[t1][d2].log";
			String clientS2File = dir + "/batchMatchingClientS2[t1][d2].log";
			String impostorRanFile = dir
					+ "/batchMatchingImpostorRandom[t1][d2].log";
			String impostorSkiFile = dir + "/batchMatchingImpostor[t1][d2].log";
			String eerRandomS1 = Formatter.f0d00(getEer(clientS1File,
					impostorRanFile).eerValue);
			String eerSkilledS1 = Formatter.f0d00(getEer(clientS1File,
					impostorSkiFile).eerValue);
			String eerRandomS2 = Formatter.f0d00(getEer(clientS2File,
					impostorRanFile).eerValue);
			String eerSkilledS2 = Formatter.f0d00(getEer(clientS2File,
					impostorSkiFile).eerValue);
			System.out.println("S1=[" + eerRandomS1 + "/" + eerSkilledS1 + "]");
			System.out.println("S2=[" + eerRandomS2 + "/" + eerSkilledS2 + "]");
		}

		if (1 == Math.random()) {
			String dir = "dist/submission/task2-ds3/documents";
			String clientS1File = dir + "/batchMatchingClientS1[t2][d3].log";
			String clientS2File = dir + "/batchMatchingClientS2[t2][d3].log";
			String impostorRanFile = dir
					+ "/batchMatchingImpostorRandom[t2][d3].log";
			String impostorSkiFile = dir + "/batchMatchingImpostor[t2][d3].log";
			String eerRandomS1 = Formatter.f0d00(getEer(clientS1File,
					impostorRanFile).eerValue);
			String eerSkilledS1 = Formatter.f0d00(getEer(clientS1File,
					impostorSkiFile).eerValue);
			String eerRandomS2 = Formatter.f0d00(getEer(clientS2File,
					impostorRanFile).eerValue);
			String eerSkilledS2 = Formatter.f0d00(getEer(clientS2File,
					impostorSkiFile).eerValue);
			System.out.println("S1=[" + eerRandomS1 + "/" + eerSkilledS1 + "]");
			System.out.println("S2=[" + eerRandomS2 + "/" + eerSkilledS2 + "]");
		}

	}
	*/

}
