package srcUmbral.umbral;

public class Score implements Comparable<Score> {
	/** Tipos de Score */
	public static final String RD = "ran";
	public static final String SK = "ski";
	public static final String TPL = "pln";
	public static final String TEST = "tst";
	public static final String COHORT = "coh";
	public static final String CLI = "cli";
	public static final String FOR = "for";

	/** Pdades del score */
	private String id;
	private double valor;
	private boolean autentica;
	private String testType;
	private boolean template;

	/**
	 * 
	 * @param id
	 * @param valor
	 * @param autentica
	 * @param trial
	 * @param testType
	 */
	public Score(String id, double valor, boolean autentica, String testType,
			boolean template) {
		setId(id);
		setValor(valor);
		setAutentica(autentica);
		setTestType(testType);
		setTemplate(template);
	}

	/**
	 * 
	 * @param id
	 * @param valor
	 * @param autentica
	 * @param trial
	 * @param testType
	 */
	public Score(String id, double valor, boolean autentica, boolean template) {
		setId(id);
		setValor(valor);
		setAutentica(autentica);
		setTemplate(template);
	}

	/**
	 * @param valor
	 * @param autentica
	 */
	public Score(double valor, boolean autentica) {
		setValor(valor);
		setAutentica(autentica);
	}

	/**
	 * 
	 * @param scores
	 * @return
	 */
	public static double[] getValues(Score[] scores) {
		double[] values = new double[scores.length];
		for (int i = 0; i < values.length; i++) {
			values[i] = scores[i].getValor();
		}
		return values;
	}

	/**
	 * Crea un array como la suma de los datos de los dos arrays recibidos
	 * 
	 * @param scoresGen
	 * @param scoresImp
	 * @return
	 */
	public static Score[] union(Score[] scoresGen, Score[] scoresImp) {

		// Array contenedor
		int genLen = (scoresGen != null) ? scoresGen.length : 0;
		int impLen = (scoresImp != null) ? scoresImp.length : 0;
		Score[] scores = new Score[genLen + impLen];
		int index = 0;

		// Aadimos los genuinos
		for (int i = 0; i < genLen; i++) {
			scores[index++] = scoresGen[i];
		}

		// Aadimos los de impostor
		for (int i = 0; i < impLen; i++) {
			scores[index++] = scoresImp[i];
		}

		// Devolvemos el array contenedor
		return scores;
	}

	/**
	 * Elimina las plantillas del array de resultados (para prueba honesta)
	 * 
	 * @param scores
	 * @param plns
	 * @return
	 */
	public static Score[] elimina(Score[] scores, String[] plns) {

		Score[] newScores = new Score[scores.length - plns.length];
		int index = 0;

		// Buscamos en todos los scores recibidos
		for (int i = 0; i < scores.length; i++) {

			// Miramos si el score es de una plantilla
			boolean esPlantilla = false;
			for (int j = 0; j < plns.length; j++) {
				if (scores[i].getId().equals(plns[j].replaceAll(".ascii", ""))) {
					esPlantilla = true;
					break;
				}
			}

			// Si no es una plantilla lo copiamos al nuevo array
			if (!esPlantilla) {
				newScores[index] = scores[i];
				index++;
			}
		}

		// Devolvemos al array de scores sin los de las plantillas
		return newScores;
	}

	/**
	 * 
	 * @param allScores
	 * @param newScores
	 * @return
	 */
	public static Score[] acumula(Score[] allScores, Score[] newScores) {
		if (allScores == null) {
			return newScores;
		}
		int n1 = allScores.length;
		int n2 = newScores.length;
		Score[] scores = new Score[n1 + n2];
		for (int i = 0; i < n1; i++) {
			scores[i] = allScores[i];
		}
		for (int i = n1; i < n1 + n2; i++) {
			scores[i] = newScores[i - n1];
		}
		return scores;
	}

	/**
	 * @param scores
	 * @param testType
	 * @return
	 */
	public static Score[] clone(Score[] scores, String testType) {
		Score[] scoresCopy = new Score[scores.length];
		for (int i = 0; i < scoresCopy.length; i++) {
			scoresCopy[i] = Score.clone(scores[i], testType);
		}
		return scoresCopy;
	}

	/**
	 * @param scores
	 * @param testType
	 * @return
	 */
	public static Score[] clone(Score[] scores) {
		Score[] scoresCopy = new Score[scores.length];
		for (int i = 0; i < scoresCopy.length; i++) {
			scoresCopy[i] = Score.clone(scores[i]);
		}
		return scoresCopy;
	}

	/**
	 * @param score
	 * @param testType
	 * @return
	 */
	private static Score clone(Score score, String testType) {
		String id = score.getId();
		double valor = score.getValor();
		boolean autentica = score.isAutentica();
		boolean template = score.isTemplate();
		return new Score(id, valor, autentica, testType, template);
	}

	/**
	 * @param score
	 * @param testType
	 * @return
	 */
	private static Score clone(Score score) {
		String id = score.getId();
		double valor = score.getValor();
		boolean autentica = score.isAutentica();
		boolean template = score.isTemplate();
		return new Score(id, valor, autentica, template);
	}

	/**
	 * 
	 * @param autentica
	 */
	public void setAutentica(boolean autentica) {
		this.autentica = autentica;
	}

	/**
	 * @return
	 */
	public boolean isAutentica() {
		return autentica;
	}

	/**
	 * @param valor
	 */
	public void setValor(double valor) {
		this.valor = valor;
	}

	/**
	 * 
	 * @return
	 */
	public double getValor() {
		return valor;
	}

	/**
	 * 
	 * @param id
	 */
	public void setId(String id) {
		this.id = id;
	}

	/**
	 * 
	 * @return
	 */
	public String getId() {
		return id;
	}

	/**
	 * @return
	 */
	public String getTestType() {
		return testType;
	}

	/**
	 * @param testType
	 */
	public void setTestType(String testType) {
		this.testType = testType;
	}

	/**
	 * @return
	 */
	public String isPlantilla() {
		return (this.isTemplate()) ? TPL : TEST;
	}

	/**
	 * @return
	 */
	public String isForgery() {
		return (this.isAutentica()) ? CLI : FOR;
	}

	/**
	 * @param template
	 */
	public void setTemplate(boolean template) {
		this.template = template;
	}

	/**
	 * @return
	 */
	public boolean isTemplate() {
		return template;
	}

	/**
	 * 
	 */
	public int compareTo(Score objScore) {
		Score otroScore = objScore;
		if (this.getValor() > otroScore.getValor())
			return +1;
		else if (this.getValor() < otroScore.getValor())
			return -1;
		return 0;
	}

	/**
	 * 
	 */
	public String toString() {
		String s = "id=" + this.id;
		s += "; valor=" + this.valor;
		s += "; autentica=" + this.autentica;
		s += "; isTemplate=" + this.template;
		s += "; testType=" + this.testType;
		return s;
	}

}
