package srcUmbral.umbral;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Scanner;

import android.content.res.Resources;
import android.util.Log;
import android.webkit.WebView.FindListener;

import es.uva.davplov.tfg_patronlibre.Point;
import es.uva.davplov.tfg_patronlibre.R;
import fastDTW.dtw.TimeWarpInfo;
import fastDTW.timeseries.TimeSeries;

import srcUmbral.modelosRegresion.*;
import srcUmbral.modelosRegresion.VarModRegLin.*;

import srcUmbral.configuracion.Configuracion;
import srcUmbral.MisExcepciones.UmbralException;

import srcUmbral.firmweb.util.ArraysUtil;
import srcUmbral.firmweb.util.FileUtil;
import srcUmbral.firmweb.util.Matematicas;
import srcUmbral.firmweb.util.Maths;



/*
 * Clase principal (entrada) de este paquete
 */
public class UmbralPrincipal {

	public double [] CalculaUmbral(double [][][] CaracFirmasEnt, double [][][] CaracFirmasCoh, 
			int sesEnt) throws UmbralException {

		double [] userThresholds = new double [4];

		// Calculamos las salidas para muestras de cohorte
		double[] dScoTplCoh = null;
		dScoTplCoh = this.testCohort(CaracFirmasEnt, CaracFirmasCoh);

		// Calculamos salidas propias mediante leave-one-out
		double[] dScoAut = this.leaveOneOut(CaracFirmasEnt);

		// Calculamos los umbrales
		if (Configuracion.RegresionModel.equals("fichero")) {
		String fich = Configuracion.RUTA_RAIZ + Configuracion.PL_FICHS_DIR + "/CoefPredLin_FRR_Biowam.txt";
		userThresholds[0] = calculaUmbralPlFarFrr(dScoTplCoh, dScoAut, fich, "Frr", 0.28);
		fich = Configuracion.RUTA_RAIZ + Configuracion.PL_FICHS_DIR + "/CoefPredLin_EERski_Biowam.txt";
		userThresholds[1] = calculaUmbralPlEER(dScoTplCoh, dScoAut, fich);
		fich = Configuracion.RUTA_RAIZ + Configuracion.PL_FICHS_DIR + "/CoefPredLin_EERran_Biowam.txt";
		userThresholds[2] = calculaUmbralPlEER(dScoTplCoh, dScoAut, fich);
		fich = Configuracion.RUTA_RAIZ + Configuracion.PL_FICHS_DIR + "/CoefPredLin_FAR_Biowam.txt";
		userThresholds[3] = calculaUmbralPlFarFrr(dScoTplCoh, dScoAut, fich, "Far", 0.1);
		} else {
			LinkedHashMap <String,Double> CoefPred = new LinkedHashMap <String,Double>();
			CoefModRegLinI coef = null;
			switch(sesEnt) {
			case 1:
				coef = new CoefModRegLinS1();
				break;
			case 2:
				coef = new CoefModRegLinS2();
				break;
			}
			int i=0;
			double [] coefValues = coef.getEER_Rn();
			for (VarEER v:VarEER.values()) {
				CoefPred.put(v.name(), coefValues[i]);
				i++;
			}
			userThresholds[0] = calculaUmbralPlEER(dScoTplCoh, dScoAut, CoefPred);
			i = 0;
			coefValues = coef.getEER_Sk();
			for (VarEER v:VarEER.values()) {
				CoefPred.put(v.name(), coefValues[i]);
				i++;
			}
			userThresholds[1] = calculaUmbralPlEER(dScoTplCoh, dScoAut, CoefPred);
			i = 0;
			coefValues = coef.getFRR20_Rn();
			for (VarFarFrr v:VarFarFrr.values()) {
				CoefPred.put(v.name(), coefValues[i]);
				i++;
			}
			userThresholds[2] = calculaUmbralPIFarFrr(dScoTplCoh, dScoAut, CoefPred, "Frr", 0.2);
			i = 0;
			coefValues = coef.getFRR30_Rn();
			for (VarFarFrr v:VarFarFrr.values()) {
				CoefPred.put(v.name(), coefValues[i]);
				i++;
			}
			userThresholds[3] = calculaUmbralPIFarFrr(dScoTplCoh, dScoAut, CoefPred, "Frr", 0.3);
		}
		
		Arrays.sort(userThresholds);
		
		// Parte annadida porque he observado que en una firma pequenna daba umbrales negativos.
		// Pongo esta solucion de momento, deducido comparando umbrales 1 y 2 de algunos usuarios y vista
		//la proporcion entre ellos...
		if (userThresholds[1] < 0) {  // Considero que hay algn problema en la generacion de umbrales. Genero error
			throw new UmbralException("ERROR al calcular umbral: segundo valor negativo");
		}
		
		if (userThresholds[0] < 0) {
			userThresholds[0] =  userThresholds[1] * 0.75;
		}

		return userThresholds;
	}

	private double calculaUmbralPIFarFrr(double[] cohs, double[] plns,
			LinkedHashMap<String, Double> CoefPred, String errType, double ref) throws UmbralException {

		// Calculo umbral
		double predic=0.0, coef;
		Eer errPriori;
		if (errType.equalsIgnoreCase("Frr")) {
			errPriori = Eer.Frr_fixed(plns,cohs,ref);
		} else {
			errPriori = Eer.Far_fixed(plns,cohs,ref);
		}

		//System.out.print("Usando para prediccion: ");
		if ((coef=CoefPred.get("OrdOrig")) != 0) {
			predic += coef;
			//System.out.print("OrdOrig ");
			//System.out.println("OrdOrig coef: " + coef);
		}
		if ( CoefPred.get("eerPriori") != null && (coef=CoefPred.get("eerPriori")) != 0) {
			predic += coef * errPriori.eerValue;
			//System.out.print("eerPriori ");
			//System.out.println("eerPriori. Valor: " + errPriori.eerValue + ", coef: " + coef);
		}
		if (CoefPred.get("eerThreshold") != null && (coef=CoefPred.get("eerThreshold")) != 0) {
			predic += coef * errPriori.eerThreshold;
			//System.out.print("eerThreshold ");
			//System.out.println("eerThreshols. Valor: " + errPriori.eerThreshold + ", coef: " + coef);
		}
		if ((coef=CoefPred.get("avPlns")) != 0) {
			predic += coef * Maths.getMean(plns);
			//System.out.print("avPlns ");
			//System.out.println("avPlns. Valor: " + Maths.getMean(plns) + ", coef: " + coef);
		}
		if ((coef=CoefPred.get("sdPlns")) != 0) {
			predic += coef * Maths.getStdDev(plns);
			//System.out.print("sdPlns ");
			//System.out.println("sdPlns. Valor: " + Maths.getStdDev(plns) + ", coef: " + coef);
		}
		if ((coef=CoefPred.get("sdpPlns")) != 0) {
			predic += coef * (Maths.getStdDev(plns)/Maths.getMean(plns));
			//System.out.print("sdpPlns ");
			//System.out.println("sdpPlns. Valor: " + (Maths.getStdDev(plns)/Maths.getMean(plns)) + ", coef: " + coef);

		}
		if ((coef=CoefPred.get("avCohs")) != 0) {
			predic += coef * Maths.getMean(cohs);
			//System.out.print("avCohs ");
		}
		if ((coef=CoefPred.get("sdCohs")) != 0) {
			predic += coef * Maths.getStdDev(cohs);
			//System.out.println("sdCohs. Valor: " + Maths.getStdDev(cohs) + ", coef: " + coef);
			//System.out.print("sdCohs ");
		}
		if ((coef=CoefPred.get("sdpCohs")) != 0) {
			predic += coef * (Maths.getStdDev(cohs)/Maths.getMean(cohs));
			//System.out.print("sdpCohs ");
		}
		if ((coef=CoefPred.get("bestFor")) != 0) {
			predic += coef * errPriori.bestRejectedForgery;
			//System.out.print("bestFor ");
		}
		if ((coef=CoefPred.get("worstGen")) != 0) {
			predic += coef * errPriori.worstAcceptedGenuine;
			//System.out.print("worstGen ");
		}
		if ((coef=CoefPred.get("eerLow")) != 0) {
			predic += coef * errPriori.eerLowerScore;
			//System.out.print("eerLow ");
		}
		if ((coef=CoefPred.get("eerUpp")) != 0) {
			predic += coef * errPriori.eerUpperScore;
			//System.out.print("eerUpp ");
			//System.out.println("eerUpp. Valor: " + errPriori.eerUpperScore + ", coef: " + coef);

		}
		if (errType.equalsIgnoreCase("Frr")) {
			if ((coef=CoefPred.get("err_otro")) != 0) {
				predic += coef * errPriori.farValue;
				//System.out.print("err_otro ");
			}
			if ((coef=CoefPred.get("errThreshold")) != 0) {
				predic += coef * errPriori.frrThreshold;
				//System.out.print("errThreshold ");
			}
		} else {
			if ((coef=CoefPred.get("err_otro")) != 0) {
				predic += coef * errPriori.frrValue;
				//System.out.print("err_otro ");
			}
			if ((coef=CoefPred.get("errThreshold")) != 0) {
				predic += coef * errPriori.farThreshold;
				//System.out.print("errThreshold ");
			}
		}
		if ((coef=CoefPred.get("perc25Plns")) != 0) {
			predic += coef * Maths.percentil(plns, 25);
			//System.out.print("perc25Plns ");
		}
		if ((coef=CoefPred.get("perc50Plns")) != 0) {
			predic += coef * Maths.percentil(plns, 50);
			//System.out.print("perc50Plns ");
		}
		if ((coef=CoefPred.get("perc75Plns")) != 0) {
			predic += coef * Maths.percentil(plns, 75);
			//System.out.print("perc75Plns ");
		}
		if ((coef=CoefPred.get("perc25cohs")) != 0) {
			predic += coef * Maths.percentil(cohs, 25);
			//System.out.print("perc25cohs ");
		}
		if ((coef=CoefPred.get("perc50cohs")) != 0) {
			predic += coef * Maths.percentil(cohs, 50);
			//System.out.print("perc50cohs ");
		}
		if ((coef=CoefPred.get("perc75cohs")) != 0) {
			predic += coef * Maths.percentil(cohs, 75);
			//System.out.print("perc75cohs ");
			//System.out.println("perc75cohs. Valor: " + Maths.percentil(cohs, 75) + ", coef: " + coef);
		}
		//System.out.println();

		return predic;
	}


	private double calculaUmbralPlEER(double[] cohs, double[] plns, String fich) throws UmbralException {
		// Cargamos los coeficientes de prediccion lineal
		LinkedHashMap <String,Double> CoefPred = new LinkedHashMap <String,Double>();
		String [] cadenas;
		try {
			Scanner in = new Scanner(new File(fich));
			while (in.hasNextInt()) {
				String linea = in.nextLine();
				cadenas = linea.split(" ");
				CoefPred.put(cadenas[1], Double.parseDouble(cadenas[2]));
			}
			in.close();
		} catch (FileNotFoundException e) {
			throw new UmbralException("ERROR al abrir el fichero de CoefPredLineal " + fich);
		}
		
		// Calculo umbral
		double predic=0.0, coef;
		Eer eerPriori = Eer.getEer(plns, cohs);
		
		//System.out.print("Usando para prediccion: ");
		if ((coef=CoefPred.get("OrdOrig")) != 0) {
			predic += coef;
			//System.out.print("OrdOrig ");
		}
		if ( CoefPred.get("eerPriori") != null && (coef=CoefPred.get("eerPriori")) != 0) {
			predic += coef * eerPriori.eerValue;
			//System.out.print("eerPriori ");
		}
		if (CoefPred.get("eerThreshold") != null && (coef=CoefPred.get("eerThreshold")) != 0) {
			predic += coef * eerPriori.eerThreshold;
			//System.out.print("eerThreshold ");
		}
		if ((coef=CoefPred.get("avPlns")) != 0) {
			predic += coef * Maths.getMean(plns);
			//System.out.print("avPlns ");
		}
		if ((coef=CoefPred.get("sdPlns")) != 0) {
			predic += coef * Maths.getStdDev(plns);
			//System.out.print("sdPlns ");
		}
		if ((coef=CoefPred.get("sdpPlns")) != 0) {
			predic += coef * (Maths.getStdDev(plns)/Maths.getMean(plns));
			//System.out.print("sdpPlns ");
		}
		if ((coef=CoefPred.get("avCohs")) != 0) {
			predic += coef * Maths.getMean(cohs);
			//System.out.print("avCohs ");
		}
		if ((coef=CoefPred.get("sdCohs")) != 0) {
			predic += coef * Maths.getStdDev(cohs);
			//System.out.print("sdCohs ");
		}
		if ((coef=CoefPred.get("sdpCohs")) != 0) {
			predic += coef * (Maths.getStdDev(cohs)/Maths.getMean(cohs));
			//System.out.print("sdpCohs ");
		}
		if ((coef=CoefPred.get("bestFor")) != 0) {
			predic += coef * eerPriori.bestRejectedForgery;
			//System.out.print("bestFor ");
		}
		if ((coef=CoefPred.get("worstGen")) != 0) {
			predic += coef * eerPriori.worstAcceptedGenuine;
			//System.out.print("worstGen ");
		}
		if ((coef=CoefPred.get("eerLow")) != 0) {
			predic += coef * eerPriori.eerLowerScore;
			//System.out.print("eerLow ");
		}
		if ((coef=CoefPred.get("eerUpp")) != 0) {
			predic += coef * eerPriori.eerUpperScore;
			//System.out.print("eerUpp ");
		}
//Falta			out.println("@attribute err_otro      real"); //Si FRR=>FAR, si FAR=>FRR en valor fijo
//Falta			out.println("@attribute errThreshold  real"); //Umbral a priori para el error fijado
		if ((coef=CoefPred.get("perc25Plns")) != 0) {
			predic += coef * Maths.percentil(plns, 25);
			//System.out.print("perc25Plns ");
		}
		if ((coef=CoefPred.get("perc50Plns")) != 0) {
			predic += coef * Maths.percentil(plns, 50);
			//System.out.print("perc50Plns ");
		}
		if ((coef=CoefPred.get("perc75Plns")) != 0) {
			predic += coef * Maths.percentil(plns, 75);
			//System.out.print("perc75Plns ");
		}
		if ((coef=CoefPred.get("perc25cohs")) != 0) {
			predic += coef * Maths.percentil(cohs, 25);
			//System.out.print("perc25cohs ");
		}
		if ((coef=CoefPred.get("perc50cohs")) != 0) {
			predic += coef * Maths.percentil(cohs, 50);
			//System.out.print("perc50cohs ");
		}
		if ((coef=CoefPred.get("perc75cohs")) != 0) {
			predic += coef * Maths.percentil(cohs, 75);
			//System.out.print("perc75cohs ");
		}
		//System.out.println();

		return predic;
	}
	
	private double calculaUmbralPlEER(double[] cohs, double[] plns, LinkedHashMap <String,Double> CoefPred) throws UmbralException {
				
		// Calculo umbral
		double predic=0.0, coef;
		Eer eerPriori = Eer.getEer(plns, cohs);
		
		//System.out.print("Usando para prediccion: ");
		if ((coef=CoefPred.get("OrdOrig")) != 0) {
			predic += coef;
			//System.out.print("OrdOrig ");
		}
		if ( CoefPred.get("eerPriori") != null && (coef=CoefPred.get("eerPriori")) != 0) {
			predic += coef * eerPriori.eerValue;
			//System.out.print("eerPriori ");
		}
		if (CoefPred.get("eerThreshold") != null && (coef=CoefPred.get("eerThreshold")) != 0) {
			predic += coef * eerPriori.eerThreshold;
			//System.out.print("eerThreshold ");
		}
		if ((coef=CoefPred.get("avPlns")) != 0) {
			predic += coef * Maths.getMean(plns);
			//System.out.print("avPlns ");
		}
		if ((coef=CoefPred.get("sdPlns")) != 0) {
			predic += coef * Maths.getStdDev(plns);
			//System.out.print("sdPlns ");
		}
		if ((coef=CoefPred.get("sdpPlns")) != 0) {
			predic += coef * (Maths.getStdDev(plns)/Maths.getMean(plns));
			//System.out.print("sdpPlns ");
		}
		if ((coef=CoefPred.get("avCohs")) != 0) {
			predic += coef * Maths.getMean(cohs);
			//System.out.print("avCohs ");
		}
		if ((coef=CoefPred.get("sdCohs")) != 0) {
			predic += coef * Maths.getStdDev(cohs);
			//System.out.print("sdCohs ");
		}
		if ((coef=CoefPred.get("sdpCohs")) != 0) {
			predic += coef * (Maths.getStdDev(cohs)/Maths.getMean(cohs));
			//System.out.print("sdpCohs ");
		}
		if ((coef=CoefPred.get("bestFor")) != 0) {
			predic += coef * eerPriori.bestRejectedForgery;
			//System.out.print("bestFor ");
		}
		if ((coef=CoefPred.get("worstGen")) != 0) {
			predic += coef * eerPriori.worstAcceptedGenuine;
			//System.out.print("worstGen ");
		}
		if ((coef=CoefPred.get("eerLow")) != 0) {
			predic += coef * eerPriori.eerLowerScore;
			//System.out.print("eerLow ");
		}
		if ((coef=CoefPred.get("eerUpp")) != 0) {
			predic += coef * eerPriori.eerUpperScore;
			//System.out.print("eerUpp ");
		}
//Falta			out.println("@attribute err_otro      real"); //Si FRR=>FAR, si FAR=>FRR en valor fijo
//Falta			out.println("@attribute errThreshold  real"); //Umbral a priori para el error fijado
		if ((coef=CoefPred.get("perc25Plns")) != 0) {
			predic += coef * Maths.percentil(plns, 25);
			//System.out.print("perc25Plns ");
		}
		if ((coef=CoefPred.get("perc50Plns")) != 0) {
			predic += coef * Maths.percentil(plns, 50);
			//System.out.print("perc50Plns ");
		}
		if ((coef=CoefPred.get("perc75Plns")) != 0) {
			predic += coef * Maths.percentil(plns, 75);
			//System.out.print("perc75Plns ");
		}
		if ((coef=CoefPred.get("perc25cohs")) != 0) {
			predic += coef * Maths.percentil(cohs, 25);
			//System.out.print("perc25cohs ");
		}
		if ((coef=CoefPred.get("perc50cohs")) != 0) {
			predic += coef * Maths.percentil(cohs, 50);
			//System.out.print("perc50cohs ");
		}
		if ((coef=CoefPred.get("perc75cohs")) != 0) {
			predic += coef * Maths.percentil(cohs, 75);
			//System.out.print("perc75cohs ");
		}
		//System.out.println();

		return predic;
	}
	
	private double calculaUmbralPlFarFrr(double[] cohs, double[] plns, String fich, String errType, double ref) throws UmbralException {
		// Cargamos los coeficientes de prediccion lineal
		LinkedHashMap <String,Double> CoefPred = new LinkedHashMap <String,Double>();
		String [] cadenas;
		try {
			Scanner in = new Scanner(new File(fich));
			while (in.hasNextInt()) {
				String linea = in.nextLine();
				cadenas = linea.split(" ");
				CoefPred.put(cadenas[1], Double.parseDouble(cadenas[2]));
			}
			in.close();
		} catch (FileNotFoundException e) {
			throw new UmbralException("ERROR al abrir el fichero de CoefPredLineal " + fich);
		}
		
		// Calculo umbral
		double predic=0.0, coef;
		Eer errPriori;
		if (errType.equalsIgnoreCase("Frr")) {
			errPriori = Eer.Frr_fixed(plns,cohs,ref);
		} else {
			errPriori = Eer.Far_fixed(plns,cohs,ref);
		}
		
		//System.out.print("Usando para prediccion: ");
		if ((coef=CoefPred.get("OrdOrig")) != 0) {
			predic += coef;
			//System.out.print("OrdOrig ");
			//System.out.println("OrdOrig coef: " + coef);
		}
		if ( CoefPred.get("eerPriori") != null && (coef=CoefPred.get("eerPriori")) != 0) {
			predic += coef * errPriori.eerValue;
			//System.out.print("eerPriori ");
			//System.out.println("eerPriori. Valor: " + errPriori.eerValue + ", coef: " + coef);
		}
		if (CoefPred.get("eerThreshold") != null && (coef=CoefPred.get("eerThreshold")) != 0) {
			predic += coef * errPriori.eerThreshold;
			//System.out.print("eerThreshold ");
			//System.out.println("eerThreshols. Valor: " + errPriori.eerThreshold + ", coef: " + coef);
		}
		if ((coef=CoefPred.get("avPlns")) != 0) {
			predic += coef * Maths.getMean(plns);
			//System.out.print("avPlns ");
			//System.out.println("avPlns. Valor: " + Maths.getMean(plns) + ", coef: " + coef);
		}
		if ((coef=CoefPred.get("sdPlns")) != 0) {
			predic += coef * Maths.getStdDev(plns);
			//System.out.print("sdPlns ");
			//System.out.println("sdPlns. Valor: " + Maths.getStdDev(plns) + ", coef: " + coef);
		}
		if ((coef=CoefPred.get("sdpPlns")) != 0) {
			predic += coef * (Maths.getStdDev(plns)/Maths.getMean(plns));
			//System.out.print("sdpPlns ");
			//System.out.println("sdpPlns. Valor: " + (Maths.getStdDev(plns)/Maths.getMean(plns)) + ", coef: " + coef);
			
		}
		if ((coef=CoefPred.get("avCohs")) != 0) {
			predic += coef * Maths.getMean(cohs);
			//System.out.print("avCohs ");
		}
		if ((coef=CoefPred.get("sdCohs")) != 0) {
			predic += coef * Maths.getStdDev(cohs);
			//System.out.println("sdCohs. Valor: " + Maths.getStdDev(cohs) + ", coef: " + coef);
			//System.out.print("sdCohs ");
		}
		if ((coef=CoefPred.get("sdpCohs")) != 0) {
			predic += coef * (Maths.getStdDev(cohs)/Maths.getMean(cohs));
			//System.out.print("sdpCohs ");
		}
		if ((coef=CoefPred.get("bestFor")) != 0) {
			predic += coef * errPriori.bestRejectedForgery;
			//System.out.print("bestFor ");
		}
		if ((coef=CoefPred.get("worstGen")) != 0) {
			predic += coef * errPriori.worstAcceptedGenuine;
			//System.out.print("worstGen ");
		}
		if ((coef=CoefPred.get("eerLow")) != 0) {
			predic += coef * errPriori.eerLowerScore;
			//System.out.print("eerLow ");
		}
		if ((coef=CoefPred.get("eerUpp")) != 0) {
			predic += coef * errPriori.eerUpperScore;
			//System.out.print("eerUpp ");
			//System.out.println("eerUpp. Valor: " + errPriori.eerUpperScore + ", coef: " + coef);
			
		}
		if (errType.equalsIgnoreCase("Frr")) {
			if ((coef=CoefPred.get("err_otro")) != 0) {
				predic += coef * errPriori.farValue;
				//System.out.print("err_otro ");
			}
			if ((coef=CoefPred.get("errThreshold")) != 0) {
				predic += coef * errPriori.frrThreshold;
				//System.out.print("errThreshold ");
			}
		} else {
			if ((coef=CoefPred.get("err_otro")) != 0) {
				predic += coef * errPriori.frrValue;
				//System.out.print("err_otro ");
			}
			if ((coef=CoefPred.get("errThreshold")) != 0) {
				predic += coef * errPriori.farThreshold;
				//System.out.print("errThreshold ");
			}
		}
		if ((coef=CoefPred.get("perc25Plns")) != 0) {
			predic += coef * Maths.percentil(plns, 25);
			//System.out.print("perc25Plns ");
		}
		if ((coef=CoefPred.get("perc50Plns")) != 0) {
			predic += coef * Maths.percentil(plns, 50);
			//System.out.print("perc50Plns ");
		}
		if ((coef=CoefPred.get("perc75Plns")) != 0) {
			predic += coef * Maths.percentil(plns, 75);
			//System.out.print("perc75Plns ");
		}
		if ((coef=CoefPred.get("perc25cohs")) != 0) {
			predic += coef * Maths.percentil(cohs, 25);
			//System.out.print("perc25cohs ");
		}
		if ((coef=CoefPred.get("perc50cohs")) != 0) {
			predic += coef * Maths.percentil(cohs, 50);
			//System.out.print("perc50cohs ");
		}
		if ((coef=CoefPred.get("perc75cohs")) != 0) {
			predic += coef * Maths.percentil(cohs, 75);
			//System.out.print("perc75cohs ");
			//System.out.println("perc75cohs. Valor: " + Maths.percentil(cohs, 75) + ", coef: " + coef);
		}
		//System.out.println();

		return predic;
	}

	private double[] leaveOneOut(double[][][] caracFirmasEnt) {
		double[] scoAut = new double[caracFirmasEnt.length];
		for (int i = 0; i < caracFirmasEnt.length; i++) {
			// Firma actual (referencia)
			double[][] ref = caracFirmasEnt[i];
			// Comparamos con el resto de firmas de la inscripcion
			double[] scoTemp = new double[caracFirmasEnt.length - 1];
			int indice = 0;
			for (int j = 0; j < caracFirmasEnt.length; j++) {
				// Saltamos la comparaci�n consigo misma
				if (i == j)
					continue;
				// Firma de prueba
				double[][] test = caracFirmasEnt[j];
				// Calculamos la distancia DTW entre ambas
				//double dtwDist = Dtw.dtwDistance(ref, test);
				//Original: double dtwDist = Dtw.distance(ref, test);
				TimeSeries firmaRef = new TimeSeries(ref, false);
				TimeSeries firmaTest = new TimeSeries(test, false);
				TimeWarpInfo info = fastDTW.dtw.FastDTW.getWarpInfoBetween(firmaRef,
						firmaTest, Configuracion.radio, Configuracion.distFn);
				double dtwDist = info.getDistance();
				// La almacenamos en el array de distancias
				scoTemp[indice] = dtwDist;
				indice++;
			}
			scoAut[i] = Matematicas.min(scoTemp);
		}
		return scoAut;
	}

	private double[] testCohort(double[][][] firmasEnt, double[][][] fichCohort)
			throws UmbralException {
		 
		
		double[] salidasCohort = new double[fichCohort.length];
		for (int i = 0; i < fichCohort.length; i++) {
			Log.d("Ficheros", "Comparo con el cohorte "+(1+i));
			double[][] caracFirma = fichCohort[i];
			// Ahora sacamos el score al comparar con cada firma de inscripcion
			double[] scores = new double[firmasEnt.length];
			for (int j = 0; j < firmasEnt.length; j++) {
				// Firma de ins
				double[][] firmaEntPro = firmasEnt[j];
				// Calculamos la distancia DTW entre ambas
				//double dtwDist = Dtw.dtwDistance(firmaEntPro, caracFirma);
				TimeSeries firmaEnt = new TimeSeries(firmaEntPro, false);
				TimeSeries firmaCh = new TimeSeries(caracFirma, false);
				TimeWarpInfo info = fastDTW.dtw.FastDTW.getWarpInfoBetween(firmaEnt,
						firmaCh, Configuracion.radio, Configuracion.distFn);
				double dtwDist = info.getDistance();
				// Originaldouble dtwDist = Dtw.distance(firmaEntPro, caracFirma);
				// La almacenamos en el array de scores
				scores[j] = dtwDist;
			}
			// Calculamos la distancia m�n. de la firma de prueba a las de
			// inscrip.
			double minScore = Matematicas.min(scores);
			// Aadimos al array final
			salidasCohort[i] = minScore;
		}
		return salidasCohort;
	}

}
